# coding: utf-8

#動作確認環境：
# Anaconda3-2.4.0 (Pytnon 3.5.1 でも確認済み)
# numpy 1.10.2 
# pandas 0.17.1
# sympy 0.7.6.1
# numdifftools 0.9.14

# 必要パッケージ（numpy, scipy, pandas, numdifftools）読み込み
from scipy.optimize import minimize
from numpy import log, exp, array, diagonal, sqrt, linalg
from pandas import read_csv
from numdifftools import Hessian

# データファイル読み込み
c = read_csv('Test.csv')

# 対数尤度関数の定義
def func(Beta):
    # 効用関数パラメータ
    ASC_Car, ASC_Bus, B_TIME, B_FemaleC = Beta
    # 選択結果判定指標
    Choice = array([(c.Choice == 1), (c.Choice == 2), (c.Choice == 3)])
    # 効用関数
    V = array([
                    ASC_Car  + B_TIME * c.TimeC + B_FemaleC * c.Female,
                    ASC_Bus  + B_TIME * c.TimeB,
                               B_TIME * c.TimeR
                ])
    # ロジットモデルの選択確率と尤度
    PD = (array([c.AvailC, c.AvailB, c.AvailR]) * exp(V)).sum(axis = 0)
    P = exp(V) / PD
    IndLL = (Choice * log(P))
    return -IndLL.sum()

# パラメータの初期値を与える
Beta0 = [0,0,0,0]

# 最尤推定の実行
output = minimize(func, Beta0, method='L-BFGS-B', tol=1e-10, options={'disp': False })

# パラメータの標準偏差・t値の計算
optBeta = output.x
fHessian = Hessian(func)
stdev = sqrt(diagonal(linalg.inv(fHessian(optBeta))))

# 結果の出力 （表記を修正）
print('Estimated Parameter:', optBeta)
print('Standard deviation: ', stdev)
print('t-value: ', optBeta/stdev)
print('Sample-size: ', len(c.index))
print('Max. / Ini. LogLL: ', -output.fun, ' / ', -func(Beta0))
print('rho-sq: ', 1 - (-output.fun) / (-func(Beta0)))
print('Adj. rho-sq: ', 1 - (-output.fun - len(optBeta)) / (-func(Beta0)))
